@@ -40,7 +40,8 @@ static gchar *_saved_session_dir = NULL;
 
 char *
 gsm_util_find_desktop_file_for_app_name (const char *name,
-                                         char      **autostart_dirs)
+                                         gboolean    look_in_saved_session,
+                                         gboolean    autostart_first)
 {
         char     *app_path;
         char    **app_dirs;
@@ -50,7 +51,7 @@ gsm_util_find_desktop_file_for_app_name (const char *name,
 
         app_path = NULL;
 
-        app_dirs = gsm_util_get_app_dirs ();
+        app_dirs = gsm_util_get_desktop_dirs (look_in_saved_session, autostart_first);
 
         key_file = g_key_file_new ();
 
@@ -70,23 +71,10 @@ gsm_util_find_desktop_file_for_app_name (const char *name,
                                    NULL);
 
         if (app_path != NULL) {
-                g_debug ("GsmUtil: found in XDG app dirs: '%s'", app_path);
+                g_debug ("GsmUtil: found in XDG dirs: '%s'", app_path);
         }
 
-        if (app_path == NULL && autostart_dirs != NULL) {
-                g_key_file_load_from_dirs (key_file,
-                                           desktop_file,
-                                           (const char **) autostart_dirs,
-                                           &app_path,
-                                           G_KEY_FILE_NONE,
-                                           NULL);
-                if (app_path != NULL) {
-                        g_debug ("GsmUtil: found in autostart dirs: '%s'", app_path);
-                }
-
-        }
-
-        /* look for gnome vender prefix */
+        /* look for gnome vendor prefix */
         if (app_path == NULL) {
                 g_free (desktop_file);
                 desktop_file = g_strdup_printf ("gnome-%s.desktop", name);
@@ -98,19 +86,7 @@ gsm_util_find_desktop_file_for_app_name (const char *name,
                                            G_KEY_FILE_NONE,
                                            NULL);
                 if (app_path != NULL) {
-                        g_debug ("GsmUtil: found in XDG app dirs: '%s'", app_path);
-                }
-        }
-
-        if (app_path == NULL && autostart_dirs != NULL) {
-                g_key_file_load_from_dirs (key_file,
-                                           desktop_file,
-                                           (const char **) autostart_dirs,
-                                           &app_path,
-                                           G_KEY_FILE_NONE,
-                                           NULL);
-                if (app_path != NULL) {
-                        g_debug ("GsmUtil: found in autostart dirs: '%s'", app_path);
+                        g_debug ("GsmUtil: found in XDG dirs: '%s'", app_path);
                 }
         }
 
@@ -268,7 +244,8 @@ gsm_util_get_app_dirs ()
 }
 
 char **
-gsm_util_get_desktop_dirs ()
+gsm_util_get_desktop_dirs (gboolean include_saved_session,
+                           gboolean autostart_first)
 {
 	char **apps;
 	char **autostart;
@@ -282,24 +259,39 @@ gsm_util_get_desktop_dirs ()
 	size = 0;
 	for (i = 0; apps[i] != NULL; i++) { size++; }
 	for (i = 0; autostart[i] != NULL; i++) { size++; }
-	size += 2; /* saved session + last NULL */
+        if (include_saved_session)
+                size += 1;
 
-	result = g_new (char *, size + 1);
+	result = g_new (char *, size + 1); /* including last NULL */
+
+        size = 0;
+
+        if (autostart_first) {
+                if (include_saved_session)
+                        result[size++] = g_strdup (gsm_util_get_saved_session_dir ());
+
+                for (i = 0; autostart[i] != NULL; i++, size++) {
+                        result[size] = autostart[i];
+                }
+                for (i = 0; apps[i] != NULL; i++, size++) {
+                        result[size] = apps[i];
+                }
+        } else {
+                for (i = 0; apps[i] != NULL; i++, size++) {
+                        result[size] = apps[i];
+                }
+                for (i = 0; autostart[i] != NULL; i++, size++) {
+                        result[size] = autostart[i];
+                }
+
+                if (include_saved_session)
+                        result[size++] = g_strdup (gsm_util_get_saved_session_dir ());
+        }
 
-	for (i = 0; apps[i] != NULL; i++) {
-		result[i] = apps[i];
-	}
 	g_free (apps);
-	size = i;
-
-	for (i = 0; autostart[i] != NULL; i++) {
-		result[size + i] = autostart[i];
-	}
 	g_free (autostart);
-	size = size + i;
 
-	result[size] = g_strdup (gsm_util_get_saved_session_dir ());
-	result[size + 1] = NULL;
+	result[size] = NULL;
 
 	return result;
 }
@@ -339,9 +331,10 @@ void
 gsm_util_init_error (gboolean    fatal,
                      const char *format, ...)
 {
-        GtkWidget *dialog;
-        char      *msg;
-        va_list    args;
+        GtkButtonsType  buttons;
+        GtkWidget      *dialog;
+        char           *msg;
+        va_list         args;
 
         va_start (args, format);
         msg = g_strdup_vprintf (format, args);
@@ -357,8 +350,17 @@ gsm_util_init_error (gboolean    fatal,
                 }
         }
 
+        if (fatal)
+                buttons = GTK_BUTTONS_NONE;
+        else
+                buttons = GTK_BUTTONS_CLOSE;
+
         dialog = gtk_message_dialog_new (NULL, 0, GTK_MESSAGE_ERROR,
-                                         GTK_BUTTONS_CLOSE, "%s", msg);
+                                         buttons, "%s", msg);
+
+        if (fatal)
+                gtk_dialog_add_button (GTK_DIALOG (dialog),
+                                       _("_Log Out"), GTK_RESPONSE_CLOSE);
 
         g_free (msg);
 
@@ -368,7 +370,10 @@ gsm_util_init_error (gboolean    fatal,
         gtk_widget_destroy (dialog);
 
         if (fatal) {
-                gtk_main_quit ();
+                if (gtk_main_level () > 0)
+                        gtk_main_quit ();
+                else
+                        exit (1);
         }
 }
 
@@ -503,3 +508,14 @@ gsm_util_setenv (const char *variable,
                 g_error_free (bus_error);
         }
 }
+
+GtkIconSize
+gsm_util_get_computer_fail_icon_size (void)
+{
+        static GtkIconSize icon_size = 0;
+
+        if (icon_size == 0)
+                icon_size = gtk_icon_size_register ("gnome-session-computer-fail", 128, 128);
+
+        return icon_size;
+}
